@@ -26,77 +26,68 @@
 #include "main_window.h"
 #include "control_box.h"
 
-static gboolean focus_in_handler(	GtkWidget *widget,
-					GtkDirectionType direction,
-					gpointer data );
-static gboolean focus_out_handler(	GtkWidget *widget,
-					GtkDirectionType direction,
-					gpointer data );
-static gboolean fs_control_enter_handler(	GtkWidget *widget,
-						GdkEvent *event,
-						gpointer data );
-static gboolean fs_control_leave_handler(	GtkWidget *widget,
-						GdkEvent *event,
-						gpointer data );
-static gboolean motion_notify_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data );
-static gboolean configure_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data );
-static gboolean hide_cursor(gpointer data);
+enum
+{
+	PROP_0,
+	PROP_USE_OPENGL,
+	N_PROPERTIES
+};
+
+struct _MainWindowPrivate
+{
+	gboolean use_opengl;
+};
+
+static void vid_area_init(MainWindow *wnd, gboolean use_opengl);
+static gboolean timeout_handler(gpointer data);
 static gboolean finalize_load_state(gpointer data);
-static GMenu *menu_btn_build_menu();
-static GMenu *open_btn_build_menu();
-static void main_window_class_init(MainWindow *wnd);
-static void main_window_init(MainWindow *wnd);
+static GMenu *menu_btn_build_menu(void);
+static GMenu *open_btn_build_menu(void);
 
-static gboolean focus_in_handler(	GtkWidget *widget,
-					GtkDirectionType direction,
-					gpointer data )
+G_DEFINE_TYPE_WITH_PRIVATE(MainWindow, main_window, GTK_TYPE_APPLICATION_WINDOW)
+
+static void main_window_set_property(	GObject *object,
+					guint property_id,
+					const GValue *value,
+					GParamSpec *pspec )
 {
-	MainWindow *wnd = data;
+	MainWindow *self = MAIN_WINDOW(object);
 
-	if(wnd->fullscreen)
+	/* PROP_USE_OPENGL can only be set once during construction */
+	if(property_id == PROP_USE_OPENGL)
 	{
-		gint width;
-		gint height;
+		self->priv->use_opengl = g_value_get_boolean(value);
 
-		gtk_window_get_size(	GTK_WINDOW(wnd->fs_control),
-					&width,
-					&height );
-
-		gtk_window_move(	GTK_WINDOW(wnd->fs_control),
-					(gdk_screen_width()/2)-(width/2),
-					gdk_screen_height()-height );
-
-		gtk_widget_show_all(wnd->fs_control);
+		vid_area_init(self, self->priv->use_opengl);
+	}
+	else
+	{
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
 	}
-
-	return TRUE;
 }
 
-static gboolean focus_out_handler(	GtkWidget *widget,
-					GtkDirectionType direction,
-					gpointer data )
+static void main_window_get_property(	GObject *object,
+					guint property_id,
+					GValue *value,
+					GParamSpec *pspec )
 {
-	MainWindow *wnd = data;
+	MainWindow *self = MAIN_WINDOW(object);
 
-	if(wnd->fullscreen)
+	if(property_id == PROP_USE_OPENGL)
 	{
-		gtk_widget_hide(wnd->fs_control);
+		g_value_set_boolean(value, self->priv->use_opengl);
+	}
+	else
+	{
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
 	}
-
-	return TRUE;
 }
 
 static gboolean fs_control_enter_handler(	GtkWidget *widget,
 						GdkEvent *event,
 						gpointer data )
 {
-	MainWindow *wnd = data;
-
-	gtk_widget_set_opacity(wnd->fs_control, 1);
+	MAIN_WINDOW(data)->fs_control_hover = TRUE;
 
 	return FALSE;
 }
@@ -105,47 +96,38 @@ static gboolean fs_control_leave_handler(	GtkWidget *widget,
 						GdkEvent *event,
 						gpointer data )
 {
-	MainWindow *wnd;
-	ControlBox *control_box;
-	GtkScaleButton *button;
-	GtkWidget *popup;
-
-	wnd = data;
-	control_box = CONTROL_BOX(wnd->control_box);
-	button = GTK_SCALE_BUTTON(control_box->volume_button);
-	popup = gtk_scale_button_get_popup(button);
-
-	if(!gtk_widget_is_visible(popup))
-	{
-		gtk_widget_set_opacity(wnd->fs_control, 0);
-	}
+	MAIN_WINDOW(data)->fs_control_hover = FALSE;
 
 	return FALSE;
 }
 
-static gboolean motion_notify_handler(	GtkWidget *widget,
-					GdkEvent *event,
-					gpointer data )
+static gboolean motion_notify_handler(GtkWidget *widget, GdkEventMotion *event)
 {
-	MainWindow *wnd;
+	MainWindow *wnd = MAIN_WINDOW(widget);
 	GdkCursor *cursor;
 
-	wnd = data;
 	cursor = gdk_cursor_new_for_display(	gdk_display_get_default(),
 						GDK_ARROW );
 
 	gdk_window_set_cursor
 		(gtk_widget_get_window(GTK_WIDGET(wnd->vid_area)), cursor);
 
+	if(wnd->fullscreen)
+	{
+		gtk_widget_show(wnd->control_box);
+	}
+
 	if(wnd->timeout_tag > 0)
 	{
 		g_source_remove(wnd->timeout_tag);
 	}
 
-	wnd->timeout_tag
-		= g_timeout_add_seconds(CURSOR_HIDE_DELAY, hide_cursor, data);
+	wnd->timeout_tag = g_timeout_add_seconds(	FS_CONTROL_HIDE_DELAY,
+							timeout_handler,
+							wnd );
 
-	return FALSE;
+	return	GTK_WIDGET_CLASS(main_window_parent_class)
+		->motion_notify_event(widget, event);
 }
 
 static gboolean configure_handler(	GtkWidget *widget,
@@ -173,17 +155,60 @@ static gboolean configure_handler(	GtkWidget *widget,
 	return FALSE;
 }
 
-static gboolean hide_cursor(gpointer data)
+static void vid_area_init(MainWindow *wnd, gboolean use_opengl)
+{
+	/* vid_area cannot be initialized more than once */
+	if(!wnd->vid_area)
+	{
+		GtkStyleContext *style_context;
+
+		wnd->vid_area =	use_opengl?
+				gtk_gl_area_new():
+				gtk_drawing_area_new();
+
+		style_context = gtk_widget_get_style_context(wnd->vid_area);
+
+		gtk_widget_add_events(wnd->vid_area, GDK_BUTTON_PRESS_MASK);
+		gtk_style_context_add_class(style_context, "gmpv-vid-area");
+
+		gtk_container_add(	GTK_CONTAINER(wnd->vid_area_overlay),
+					wnd->vid_area );
+
+		gtk_paned_pack1(	GTK_PANED(wnd->vid_area_paned),
+					wnd->vid_area_overlay,
+					TRUE,
+					TRUE );
+	}
+}
+
+static gboolean timeout_handler(gpointer data)
 {
 	MainWindow *wnd;
-	GdkCursor *cursor;
+	ControlBox *control_box;
+	GtkScaleButton *button;
+	GtkWidget *popup;
 
 	wnd = data;
-	cursor = gdk_cursor_new_for_display(	gdk_display_get_default(),
-						GDK_BLANK_CURSOR );
+	control_box = CONTROL_BOX(wnd->control_box);
+	button = GTK_SCALE_BUTTON(control_box->volume_button);
+	popup = gtk_scale_button_get_popup(button);
 
-	gdk_window_set_cursor
-		(gtk_widget_get_window(GTK_WIDGET(wnd->vid_area)), cursor);
+
+	if(wnd->fullscreen
+	&& !wnd->fs_control_hover
+	&& !gtk_widget_is_visible(popup))
+	{
+		GdkWindow *window;
+		GdkCursor *cursor;
+
+		window = gtk_widget_get_window(GTK_WIDGET(wnd->vid_area));
+
+		cursor = gdk_cursor_new_for_display
+				(gdk_display_get_default(), GDK_BLANK_CURSOR);
+
+		gdk_window_set_cursor(window, cursor);
+		gtk_widget_hide(wnd->control_box);
+	}
 
 	wnd->timeout_tag = 0;
 
@@ -202,7 +227,7 @@ static gboolean finalize_load_state(gpointer data)
 		wnd->playlist_visible = wnd->init_playlist_visible;
 
 		gtk_widget_set_visible(	wnd->playlist,
-					wnd->init_playlist_visible);
+					wnd->init_playlist_visible );
 	}
 
 	return FALSE;
@@ -220,7 +245,6 @@ static GMenu *menu_btn_build_menu()
 	GMenuItem *load_sub_menu_item;
 	GMenuItem *playlist_toggle_menu_item;
 	GMenuItem *playlist_save_menu_item;
-	GMenuItem *fullscreen_menu_item;
 	GMenuItem *normal_size_menu_item;
 	GMenuItem *double_size_menu_item;
 	GMenuItem *half_size_menu_item;
@@ -248,9 +272,6 @@ static GMenu *menu_btn_build_menu()
 	load_sub_menu_item
 		= g_menu_item_new(_("_Load Subtitle"), "app.loadsub");
 
-	fullscreen_menu_item
-		= g_menu_item_new(_("_Fullscreen"), "app.fullscreen");
-
 	normal_size_menu_item
 		= g_menu_item_new(_("_Normal Size"), "app.normalsize");
 
@@ -263,7 +284,6 @@ static GMenu *menu_btn_build_menu()
 	g_menu_append_item(playlist_submenu, playlist_toggle_menu_item);
 	g_menu_append_item(playlist_submenu, playlist_save_menu_item);
 	g_menu_append_item(sub_submenu, load_sub_menu_item);
-	g_menu_append_item(view_submenu, fullscreen_menu_item);
 	g_menu_append_item(view_submenu, normal_size_menu_item);
 	g_menu_append_item(view_submenu, double_size_menu_item);
 	g_menu_append_item(view_submenu, half_size_menu_item);
@@ -375,10 +395,18 @@ void main_window_load_state(MainWindow *wnd)
 	g_free(config_file);
 }
 
-static void main_window_class_init(MainWindow *wnd)
+static void main_window_class_init(MainWindowClass *klass)
 {
+	GObjectClass *obj_class = G_OBJECT_CLASS(klass);
+	GtkWidgetClass *wgt_class = GTK_WIDGET_CLASS(klass);
+	GParamSpec *pspec = NULL;
+
+	obj_class->set_property = main_window_set_property;
+	obj_class->get_property = main_window_get_property;
+	wgt_class->motion_notify_event = motion_notify_handler;
+
 	g_signal_new(	"mpv-init",
-			MAIN_WINDOW_TYPE,
+			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
 			0,
 			NULL,
@@ -388,7 +416,7 @@ static void main_window_class_init(MainWindow *wnd)
 			0 );
 
 	g_signal_new(	"mpv-playback-restart",
-			MAIN_WINDOW_TYPE,
+			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
 			0,
 			NULL,
@@ -398,7 +426,7 @@ static void main_window_class_init(MainWindow *wnd)
 			0 );
 
 	g_signal_new(	"mpv-prop-change",
-			MAIN_WINDOW_TYPE,
+			G_TYPE_FROM_CLASS(klass),
 			G_SIGNAL_RUN_FIRST,
 			0,
 			NULL,
@@ -407,14 +435,24 @@ static void main_window_class_init(MainWindow *wnd)
 			G_TYPE_NONE,
 			1,
 			G_TYPE_STRING );
+
+	pspec = g_param_spec_boolean
+		(	"use-opengl",
+			"Use OpenGL",
+			"Whether or not to set up video area for opengl-cb",
+			FALSE,
+			G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE );
+
+	g_object_class_install_property(obj_class, PROP_USE_OPENGL, pspec);
 }
 
 static void main_window_init(MainWindow *wnd)
 {
-	GdkRGBA vid_area_bg_color;
-
+	/* wnd->vid_area will be initialized when use-opengl property is set */
+	wnd->priv = main_window_get_instance_private(wnd);
 	wnd->fullscreen = FALSE;
 	wnd->playlist_visible = FALSE;
+	wnd->fs_control_hover = FALSE;
 	wnd->playlist_width = PLAYLIST_DEFAULT_WIDTH;
 	wnd->timeout_tag = 0;
 	wnd->init_width = MAIN_WINDOW_DEFAULT_WIDTH;
@@ -426,19 +464,14 @@ static void main_window_init(MainWindow *wnd)
 	wnd->menu_hdr_btn = NULL;
 	wnd->main_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	wnd->vid_area_paned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
-	wnd->vid_area = gtk_drawing_area_new();
-	wnd->fs_control = gtk_window_new(GTK_WINDOW_POPUP);
+	wnd->vid_area_overlay = gtk_overlay_new();
 	wnd->control_box = control_box_new();
 	wnd->playlist = playlist_widget_new();
 
-	gdk_rgba_parse(&vid_area_bg_color, VID_AREA_BG_COLOR);
-
-	gtk_widget_add_events(	wnd->fs_control,
+	gtk_widget_add_events(	wnd->vid_area_overlay,
 				GDK_ENTER_NOTIFY_MASK
 				|GDK_LEAVE_NOTIFY_MASK );
 
-	gtk_widget_add_events(wnd->vid_area, GDK_BUTTON_PRESS_MASK);
-
 	gtk_header_bar_set_show_close_button(	GTK_HEADER_BAR(wnd->header_bar),
 						TRUE );
 
@@ -452,16 +485,9 @@ static void main_window_init(MainWindow *wnd)
 					MAIN_WINDOW_DEFAULT_WIDTH,
 					MAIN_WINDOW_DEFAULT_HEIGHT );
 
-	gtk_widget_override_background_color(	wnd->vid_area,
-						GTK_STATE_NORMAL,
-						&vid_area_bg_color);
-
 	gtk_box_pack_start
 		(GTK_BOX(wnd->main_box), wnd->vid_area_paned, TRUE, TRUE, 0);
 
-	gtk_paned_pack1
-		(GTK_PANED(wnd->vid_area_paned), wnd->vid_area, TRUE, TRUE);
-
 	gtk_paned_pack2
 		(GTK_PANED(wnd->vid_area_paned), wnd->playlist, FALSE, TRUE);
 
@@ -471,82 +497,45 @@ static void main_window_init(MainWindow *wnd)
 	gtk_container_add
 		(GTK_CONTAINER(wnd), wnd->main_box);
 
-	g_signal_connect(	wnd,
-				"focus-in-event",
-				G_CALLBACK(focus_in_handler),
-				wnd );
-
-	g_signal_connect(	wnd,
-				"focus-out-event",
-				G_CALLBACK(focus_out_handler),
-				wnd );
-
-	g_signal_connect(	wnd->fs_control,
+	g_signal_connect(	wnd->vid_area_overlay,
 				"enter-notify-event",
 				G_CALLBACK(fs_control_enter_handler),
 				wnd );
 
-	g_signal_connect(	wnd->fs_control,
+	g_signal_connect(	wnd->vid_area_overlay,
 				"leave-notify-event",
 				G_CALLBACK(fs_control_leave_handler),
 				wnd );
-
-	g_signal_connect(	wnd,
-				"motion-notify-event",
-				G_CALLBACK(motion_notify_handler),
-				wnd );
 }
 
-GtkWidget *main_window_new(GtkApplication *app)
+GtkWidget *main_window_new(GtkApplication *app, gboolean use_opengl)
 {
 	return GTK_WIDGET(g_object_new(	main_window_get_type(),
 					"application", app,
+					"use-opengl", use_opengl,
 					NULL ));
 }
 
-GType main_window_get_type()
-{
-	static GType wnd_type = 0;
-
-	if(wnd_type == 0)
-	{
-		const GTypeInfo wnd_info
-			= {	sizeof(MainWindowClass),
-				NULL,
-				NULL,
-				(GClassInitFunc)main_window_class_init,
-				NULL,
-				NULL,
-				sizeof(MainWindow),
-				0,
-				(GInstanceInitFunc)main_window_init };
-
-		wnd_type = g_type_register_static
-				(	GTK_TYPE_APPLICATION_WINDOW,
-					"MainWindow",
-					&wnd_info,
-					0 );
-	}
-
-	return wnd_type;
-}
-
 void main_window_toggle_fullscreen(MainWindow *wnd)
 {
 	ControlBox *control_box = CONTROL_BOX(wnd->control_box);
 	GtkContainer* main_box = GTK_CONTAINER(wnd->main_box);
-	GtkContainer* fs_control = GTK_CONTAINER(wnd->fs_control);
+	GtkContainer *overlay = GTK_CONTAINER(wnd->vid_area_overlay);
 
 	if(wnd->fullscreen)
 	{
+		gtk_widget_set_halign(wnd->control_box, GTK_ALIGN_FILL);
+		gtk_widget_set_valign(wnd->control_box, GTK_ALIGN_FILL);
+		gtk_widget_set_size_request(wnd->control_box, -1, -1);
+
 		g_object_ref(wnd->control_box);
-		gtk_container_remove(fs_control, wnd->control_box);
+		gtk_container_remove(overlay, wnd->control_box);
 		gtk_container_add(main_box, wnd->control_box);
 		g_object_unref(wnd->control_box);
 
 		control_box_set_fullscreen_state(control_box, FALSE);
 		gtk_window_unfullscreen(GTK_WINDOW(wnd));
-		gtk_widget_hide(wnd->fs_control);
+		gtk_widget_show(wnd->control_box);
 
 		if(main_window_get_csd_enabled(wnd))
 		{
@@ -569,22 +558,33 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 	else
 	{
 		GdkScreen *screen;
+		GdkWindow *window;
+		GdkRectangle monitor_geom;
 		gint width;
-		gint height;
+		gint monitor;
 
 		screen = gtk_window_get_screen(GTK_WINDOW(wnd));
-		width = gdk_screen_width()/2;
+		window = gtk_widget_get_window(GTK_WIDGET(wnd));
+		monitor = gdk_screen_get_monitor_at_window(screen, window);
+
+		gdk_screen_get_monitor_geometry(screen, monitor, &monitor_geom);
+
+		width = monitor_geom.width/2;
+
+		gtk_widget_set_halign(wnd->control_box, GTK_ALIGN_CENTER);
+		gtk_widget_set_valign(wnd->control_box, GTK_ALIGN_END);
+		gtk_widget_set_size_request(wnd->control_box, width, -1);
 
 		g_object_ref(wnd->control_box);
 		gtk_container_remove(main_box, wnd->control_box);
-		gtk_container_add(fs_control, wnd->control_box);
+		gtk_overlay_add_overlay(GTK_OVERLAY(overlay), wnd->control_box);
 		g_object_unref(wnd->control_box);
 
 		control_box_set_fullscreen_state(control_box, TRUE);
 		gtk_window_fullscreen(GTK_WINDOW(wnd));
-		gtk_window_set_screen(GTK_WINDOW(wnd->fs_control), screen);
-		gtk_widget_show(wnd->fs_control);
-		gtk_widget_set_opacity(wnd->fs_control, 0);
+		gtk_window_present(GTK_WINDOW(wnd));
+		gtk_widget_hide(wnd->control_box);
+		timeout_handler(wnd);
 
 		if(main_window_get_csd_enabled(wnd))
 		{
@@ -602,21 +602,6 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 			gtk_widget_hide(wnd->playlist);
 		}
 
-		gtk_window_get_size(	GTK_WINDOW(wnd->fs_control),
-					NULL,
-					&height );
-
-		gtk_window_resize(	GTK_WINDOW(wnd->fs_control),
-					width,
-					height );
-
-		gtk_window_move(	GTK_WINDOW(wnd->fs_control),
-					(gdk_screen_width()/2)-(width/2),
-					gdk_screen_height()-height );
-
-		gtk_window_set_transient_for(	GTK_WINDOW(wnd->fs_control),
-						GTK_WINDOW(wnd) );
-
 		wnd->fullscreen = TRUE;
 	}
 }
@@ -639,6 +624,11 @@ gint main_window_get_height_margin(MainWindow *wnd)
 		- gtk_widget_get_allocated_height(wnd->vid_area);
 }
 
+gboolean main_window_get_use_opengl(MainWindow *wnd)
+{
+	return wnd->priv->use_opengl;
+}
+
 void main_window_enable_csd(MainWindow *wnd)
 {
 	GIcon *open_icon;
