@@ -37,6 +37,7 @@ static void handle_autofit_opt(gmpv_handle *ctx);
 static void handle_msg_level_opt(gmpv_handle *ctx);
 static void handle_property_change_event(	gmpv_handle *ctx,
 						mpv_event_property* prop);
+static void opengl_callback(void *cb_ctx);
 
 static void parse_dim_string(	gmpv_handle *ctx,
 				const gchar *mpv_geom_str,
@@ -64,7 +65,7 @@ static void parse_dim_string(	gmpv_handle *ctx,
 		gdouble multiplier = -1;
 		gint value = -1;
 
-		value = g_ascii_strtoll(tokens[i], NULL, 0);
+		value = (gint)g_ascii_strtoll(tokens[i], NULL, 0);
 
 		if(tokens[i][strnlen(tokens[i], 256)-1] == '%')
 		{
@@ -79,7 +80,7 @@ static void parse_dim_string(	gmpv_handle *ctx,
 			}
 			else
 			{
-				*width = multiplier*screen_width;
+				*width = (gint)(multiplier*screen_width);
 			}
 		}
 		else if(i == 1)
@@ -90,7 +91,7 @@ static void parse_dim_string(	gmpv_handle *ctx,
 			}
 			else
 			{
-				*height = multiplier*screen_height;
+				*height = (gint)(multiplier*screen_height);
 			}
 		}
 	}
@@ -158,8 +159,11 @@ static void handle_autofit_opt(gmpv_handle *ctx)
 						&autofit_width,
 						&autofit_height );
 
-			width_ratio = (gdouble)autofit_width/vid_width;
-			height_ratio = (gdouble)autofit_height/vid_height;
+			width_ratio =	(gdouble)autofit_width/
+					(gdouble)vid_width;
+
+			height_ratio =	(gdouble)autofit_height/
+					(gdouble)vid_height;
 		}
 
 		if(rc >= 0 && width_ratio > 0 && height_ratio > 0)
@@ -335,6 +339,16 @@ static void handle_property_change_event(	gmpv_handle *ctx,
 	}
 }
 
+static void opengl_callback(void *cb_ctx)
+{
+	gmpv_handle *ctx = cb_ctx;
+
+	if(ctx->opengl_ctx)
+	{
+		gtk_gl_area_queue_render(GTK_GL_AREA(ctx->gui->vid_area));
+	}
+}
+
 void mpv_wakeup_callback(void *data)
 {
 	g_idle_add((GSourceFunc)mpv_handle_event, data);
@@ -392,13 +406,11 @@ void mpv_check_error(int status)
 
 	if(status < 0)
 	{
-		size = backtrace(array, 10);
+		size = (size_t)backtrace(array, 10);
 
-		fprintf(	stderr,
-				"MPV API error: %s\n",
-				mpv_error_string(status) );
+		g_critical("MPV API error: %s\n", mpv_error_string(status));
 
-		backtrace_symbols_fd(array, size, STDERR_FILENO);
+		backtrace_symbols_fd(array, (int)size, STDERR_FILENO);
 
 		exit(EXIT_FAILURE);
 	}
@@ -605,7 +617,8 @@ void mpv_load_gui_update(gmpv_handle *ctx)
 				&playlist_pos) >= 0)
 	{
 		playlist_widget_set_indicator_pos
-			(PLAYLIST_WIDGET(ctx->gui->playlist), playlist_pos);
+			(	PLAYLIST_WIDGET(ctx->gui->playlist),
+				(gint)playlist_pos );
 	}
 
 	if(mpv_get_property(	ctx->mpv_ctx,
@@ -630,7 +643,7 @@ void mpv_load_gui_update(gmpv_handle *ctx)
 				MPV_FORMAT_DOUBLE,
 				&length) >= 0)
 	{
-		control_box_set_seek_bar_length(control_box, length);
+		control_box_set_seek_bar_length(control_box, (gint)length);
 	}
 
 	control_box_set_playing_state(control_box, !ctx->paused);
@@ -646,7 +659,7 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 		gchar *opt_end = strstr(opt_begin, " --");
 		gchar *token;
 		gchar *token_arg;
-		gint token_size;
+		gsize token_size;
 
 		/* Point opt_end to the end of the input string if the current
 		 * option is the last one.
@@ -662,7 +675,7 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 		while(	--opt_end != opt_begin
 			&& (*opt_end == ' ' || *opt_end == '\n') );
 
-		token_size = opt_end-opt_begin;
+		token_size = (gsize)(opt_end-opt_begin);
 		token = g_malloc(token_size);
 
 		strncpy(token, opt_begin+2, token_size-1);
@@ -678,9 +691,10 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 		}
 		else
 		{
-			/* Default to "yes" if option has no explicit argument
+			/* Default to empty string if there is no explicit
+			 * argument
 			 */
-			token_arg = "yes";
+			token_arg = "";
 		}
 
 		/* Failing to apply extra options is non-fatal */
@@ -688,8 +702,7 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 		{
 			fail_count++;
 
-			fprintf(	stderr,
-					"Failed to apply option: --%s=%s\n",
+			g_warning(	"Failed to apply option: --%s=%s\n",
 					token,
 					token_arg );
 		}
@@ -707,7 +720,7 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 	return fail_count*(-1);
 }
 
-void mpv_init(gmpv_handle *ctx, gint64 vid_area_wid)
+void mpv_init(gmpv_handle *ctx)
 {
 	gboolean mpvconf_enable = FALSE;
 	gchar *config_dir = get_config_dir_path();
@@ -734,6 +747,10 @@ void mpv_init(gmpv_handle *ctx, gint64 vid_area_wid)
 						"title",
 						"${media-title}" ));
 
+	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
+						"ytdl",
+						"yes" ));
+
 	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
 						"input-cursor",
 						"no" ));
@@ -758,10 +775,13 @@ void mpv_init(gmpv_handle *ctx, gint64 vid_area_wid)
 						"screenshot-template",
 						screenshot_template ));
 
-	mpv_check_error(mpv_set_option(	ctx->mpv_ctx,
-					"wid",
-					MPV_FORMAT_INT64,
-					&vid_area_wid ));
+	if(!main_window_get_use_opengl(ctx->gui))
+	{
+		mpv_check_error(mpv_set_option(	ctx->mpv_ctx,
+						"wid",
+						MPV_FORMAT_INT64,
+						&ctx->vid_area_wid ));
+	}
 
 	mpv_check_error(mpv_observe_property(	ctx->mpv_ctx,
 						0,
@@ -803,16 +823,44 @@ void mpv_init(gmpv_handle *ctx, gint64 vid_area_wid)
 		show_error_dialog(ctx, NULL, msg);
 	}
 
+	if(main_window_get_use_opengl(ctx->gui))
+	{
+		mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
+							"vo",
+							"opengl-cb" ));
+
+	}
+
 	mpv_check_error(mpv_initialize(ctx->mpv_ctx));
 	handle_msg_level_opt(ctx);
 	g_signal_emit_by_name(ctx->gui, "mpv-init");
 
+	ctx->opengl_ctx = mpv_get_sub_api(ctx->mpv_ctx, MPV_SUB_API_OPENGL_CB);
+
+	mpv_opengl_cb_set_update_callback(	ctx->opengl_ctx,
+						opengl_callback,
+						ctx );
+
 	g_free(config_dir);
 	g_free(mpvconf);
 	g_free(mpvopt);
 	g_free(screenshot_template);
 }
 
+void mpv_quit(gmpv_handle *ctx)
+{
+	if(gtk_widget_get_realized(ctx->gui->vid_area)
+	&& main_window_get_use_opengl(ctx->gui))
+	{
+		gtk_gl_area_make_current(GTK_GL_AREA(ctx->gui->vid_area));
+		mpv_opengl_cb_uninit_gl(ctx->opengl_ctx);
+
+		ctx->opengl_ready = FALSE;
+	}
+
+	mpv_terminate_destroy(ctx->mpv_ctx);
+}
+
 void mpv_load(	gmpv_handle *ctx,
 		const gchar *uri,
 		gboolean append,
